defmodule DecentralisedBookIndex.Metadata.BookId do
  use Ash.Resource,
    otp_app: :decentralised_book_index,
    domain: DecentralisedBookIndex.Metadata,
    data_layer: AshPostgres.DataLayer,
    extensions: [AshJsonApi.Resource],
    authorizers: [Ash.Policy.Authorizer]

  alias DecentralisedBookIndex.Metadata

  json_api do
    type "book_id"
  end

  postgres do
    table "book_ids"
    repo DecentralisedBookIndex.Repo

    references do
      reference :book, index?: true, on_delete: :delete
    end

    custom_indexes do
      index "type gin_trgm_ops", name: "book_id_type_gin_index", using: "GIN"
      index "bid gin_trgm_ops", name: "book_id_bid_gin_index", using: "GIN"
    end
  end

  resource do
    description "A book's identifier."
  end

  actions do
    defaults [:destroy]

    create :create do
      primary? true
      accept [:id, :type, :bid, :order]
    end

    update :update do
      primary? true
      accept [:id, :type, :bid, :order]
    end

    read :read do
      description "Return a list of BookIds."
      primary? true
    end

    read :by_id do
      description "Return the BookId by its id."
      argument :id, :uuid, allow_nil?: false
      get? true
      filter expr(id == ^arg(:id))
    end
  end

  policies do
    bypass actor_attribute_equals(:role, :admin) do
      authorize_if always()
    end

    policy action_type(:read) do
      authorize_if always()
    end

    policy action_type(:create) do
      authorize_if actor_attribute_equals(:role, :moderator)
    end

    policy action_type(:update) do
      authorize_if actor_attribute_equals(:role, :moderator)
    end

    policy action_type(:destroy) do
      authorize_if actor_attribute_equals(:role, :moderator)
    end
  end

  attributes do
    uuid_primary_key :id, writable?: true

    attribute :type, :string do
      allow_nil? false
      public? true
    end

    attribute :bid, :string do
      allow_nil? false
      public? true
    end

    attribute :order, :integer do
      allow_nil? false
      public? true
    end

    timestamps()
  end

  relationships do
    belongs_to :dbi_server, Metadata.DBIServer

    belongs_to :book, Metadata.Book do
      allow_nil? true
    end
  end
end
